@@ -20,6 +20,7 @@
 # software solely pursuant to the terms of the relevant commercial agreement.
 
 import string
+import warnings
 from collections import defaultdict
 
 import sqlalchemy as sa
@@ -178,6 +179,22 @@ class CrateDDLCompiler(compiler.DDLCompiler):
                 ', '.join(sorted(table_opts)))
         return special_options
 
+    def visit_foreign_key_constraint(self, constraint, **kw):
+        """
+        CrateDB does not support foreign key constraints.
+        """
+        warnings.warn("CrateDB does not support foreign key constraints, "
+                      "they will be omitted when generating DDL statements.")
+        return None
+
+    def visit_unique_constraint(self, constraint, **kw):
+        """
+        CrateDB does not support unique key constraints.
+        """
+        warnings.warn("CrateDB does not support unique constraints, "
+                      "they will be omitted when generating DDL statements.")
+        return None
+
 
 class CrateTypeCompiler(compiler.GenericTypeCompiler):
 
@@ -292,3 +309,10 @@ class CrateCompiler(compiler.SQLCompiler):
         Generate OFFSET / LIMIT clause, PostgreSQL-compatible.
         """
         return PGCompiler.limit_clause(self, select, **kw)
+
+    def for_update_clause(self, select, **kw):
+        # CrateDB does not support the `INSERT ... FOR UPDATE` clause.
+        # See https://github.com/crate/crate-python/issues/577.
+        warnings.warn("CrateDB does not support the 'INSERT ... FOR UPDATE' clause, "
+                      "it will be omitted when generating SQL statements.")
+        return ''
