@@ -18,20 +18,32 @@
 # However, if you have executed another commercial license agreement
 # with Crate these terms will supersede the license and you may use the
 # software solely pursuant to the terms of the relevant commercial agreement.
+import warnings
 from textwrap import dedent
-from unittest import mock, skipIf
+from unittest import mock, skipIf, TestCase
+from unittest.mock import MagicMock, patch
 
+from crate.client.cursor import Cursor
 from crate.client.sqlalchemy.compiler import crate_before_execute
 
 import sqlalchemy as sa
 from sqlalchemy.sql import text, Update
 
+from crate.testing.util import ExtraAssertions
+
+try:
+    from sqlalchemy.orm import declarative_base
+except ImportError:
+    from sqlalchemy.ext.declarative import declarative_base
+
 from crate.client.sqlalchemy.sa_version import SA_VERSION, SA_1_4, SA_2_0
 from crate.client.sqlalchemy.types import ObjectType
 from crate.client.test_util import ParametrizedTestCase
 
+from crate.testing.settings import crate_host
 
-class SqlAlchemyCompilerTest(ParametrizedTestCase):
+
+class SqlAlchemyCompilerTest(ParametrizedTestCase, ExtraAssertions):
 
     def setUp(self):
         self.crate_engine = sa.create_engine('crate://')
@@ -244,3 +256,179 @@ class SqlAlchemyCompilerTest(ParametrizedTestCase):
             mock.call(mock.ANY, 'INSERT INTO mytable (name) VALUES (?), (?)', ('foo_2', 'foo_3'), None),
             mock.call(mock.ANY, 'INSERT INTO mytable (name) VALUES (?)', ('foo_4', ), None),
         ])
+
+    def test_for_update(self):
+        """
+        Verify the `CrateCompiler.for_update_clause` method to
+        omit the clause, since CrateDB does not support it.
+        """
+
+        with warnings.catch_warnings(record=True) as w:
+
+            # By default, warnings from a loop will only be emitted once.
+            # This scenario tests exactly this behaviour, to verify logs
+            # don't get flooded.
+            warnings.simplefilter("once")
+
+            selectable = self.mytable.select().with_for_update()
+            _ = str(selectable.compile(bind=self.crate_engine))
+
+            selectable = self.mytable.select().with_for_update()
+            statement = str(selectable.compile(bind=self.crate_engine))
+
+        # Verify SQL statement.
+        self.assertEqual(statement, "SELECT mytable.name, mytable.data \nFROM mytable")
+
+        # Verify if corresponding warning is emitted, once.
+        self.assertEqual(len(w), 1)
+        self.assertIsSubclass(w[-1].category, UserWarning)
+        self.assertIn("CrateDB does not support the 'INSERT ... FOR UPDATE' clause, "
+                      "it will be omitted when generating SQL statements.", str(w[-1].message))
+
+
+FakeCursor = MagicMock(name='FakeCursor', spec=Cursor)
+
+
+class CompilerTestCase(TestCase):
+    """
+    A base class for providing mocking infrastructure to validate the DDL compiler.
+    """
+
+    def setUp(self):
+        self.engine = sa.create_engine(f"crate://{crate_host}")
+        self.metadata = sa.MetaData(schema="testdrive")
+        self.session = sa.orm.Session(bind=self.engine)
+        self.setup_mock()
+
+    def setup_mock(self):
+        """
+        Set up a fake cursor, in order to intercept query execution.
+        """
+
+        self.fake_cursor = MagicMock(name="fake_cursor")
+        FakeCursor.return_value = self.fake_cursor
+
+        self.executed_statement = None
+        self.fake_cursor.execute = self.execute_wrapper
+
+    def execute_wrapper(self, query, *args, **kwargs):
+        """
+        Receive the SQL query expression, and store it.
+        """
+        self.executed_statement = query
+        return self.fake_cursor
+
+
+@patch('crate.client.connection.Cursor', FakeCursor)
+class SqlAlchemyDDLCompilerTest(CompilerTestCase, ExtraAssertions):
+    """
+    Verify a few scenarios regarding the DDL compiler.
+    """
+
+    def test_ddl_with_foreign_keys(self):
+        """
+        Verify the CrateDB dialect properly ignores foreign key constraints.
+        """
+
+        Base = declarative_base(metadata=self.metadata)
+
+        class RootStore(Base):
+            """The main store."""
+
+            __tablename__ = "root"
+
+            id = sa.Column(sa.Integer, primary_key=True)
+            name = sa.Column(sa.String)
+
+            items = sa.orm.relationship(
+                "ItemStore",
+                back_populates="root",
+                passive_deletes=True,
+            )
+
+        class ItemStore(Base):
+            """The auxiliary store."""
+
+            __tablename__ = "item"
+
+            id = sa.Column(sa.Integer, primary_key=True)
+            name = sa.Column(sa.String)
+            root_id = sa.Column(
+                sa.Integer,
+                sa.ForeignKey(
+                    f"{RootStore.__tablename__}.id",
+                    ondelete="CASCADE",
+                ),
+            )
+            root = sa.orm.relationship(RootStore, back_populates="items")
+
+        with warnings.catch_warnings(record=True) as w:
+
+            # Cause all warnings to always be triggered.
+            warnings.simplefilter("always")
+
+            # Verify SQL DDL statement.
+            self.metadata.create_all(self.engine, tables=[RootStore.__table__], checkfirst=False)
+            self.assertEqual(self.executed_statement, dedent("""
+                CREATE TABLE testdrive.root (
+                \tid INT NOT NULL, 
+                \tname STRING, 
+                \tPRIMARY KEY (id)
+                )
+    
+            """))  # noqa: W291, W293
+
+            # Verify SQL DDL statement.
+            self.metadata.create_all(self.engine, tables=[ItemStore.__table__], checkfirst=False)
+            self.assertEqual(self.executed_statement, dedent("""
+                CREATE TABLE testdrive.item (
+                \tid INT NOT NULL, 
+                \tname STRING, 
+                \troot_id INT, 
+                \tPRIMARY KEY (id)
+                )
+    
+            """))  # noqa: W291, W293
+
+        # Verify if corresponding warning is emitted.
+        self.assertEqual(len(w), 1)
+        self.assertIsSubclass(w[-1].category, UserWarning)
+        self.assertIn("CrateDB does not support foreign key constraints, "
+                      "they will be omitted when generating DDL statements.", str(w[-1].message))
+
+    def test_ddl_with_unique_key(self):
+        """
+        Verify the CrateDB dialect properly ignores unique key constraints.
+        """
+
+        Base = declarative_base(metadata=self.metadata)
+
+        class FooBar(Base):
+            """The entity."""
+
+            __tablename__ = "foobar"
+
+            id = sa.Column(sa.Integer, primary_key=True)
+            name = sa.Column(sa.String, unique=True)
+
+        with warnings.catch_warnings(record=True) as w:
+
+            # Cause all warnings to always be triggered.
+            warnings.simplefilter("always")
+
+            # Verify SQL DDL statement.
+            self.metadata.create_all(self.engine, tables=[FooBar.__table__], checkfirst=False)
+            self.assertEqual(self.executed_statement, dedent("""
+                CREATE TABLE testdrive.foobar (
+                \tid INT NOT NULL, 
+                \tname STRING, 
+                \tPRIMARY KEY (id)
+                )
+    
+            """))  # noqa: W291, W293
+
+        # Verify if corresponding warning is emitted.
+        self.assertEqual(len(w), 1)
+        self.assertIsSubclass(w[-1].category, UserWarning)
+        self.assertIn("CrateDB does not support unique constraints, "
+                      "they will be omitted when generating DDL statements.", str(w[-1].message))
